#!/usr/bin/env python3

name    = 'HRlo'
version = '0.7.14'

description = 'HR manager'
long_description = 'HR manager'

url = 'https://github.com/so07/HRlo'

author='so07'
author_email=''

maintainer=author
